<?php
/**
 * Plugin Name: EasyToolsWP Shortlink Generator
 * Description: A plugin to create shortlinks that redirect to other URLs with categories.
 * Version: 1.9
 * Author: EasyToolsWP
 */

// Register custom post type
function register_shortlink_post_type() {
    $labels = array(
        'name'               => _x('Shortlinks', 'post type general name', 'textdomain'),
        'singular_name'      => _x('Shortlink', 'post type singular name', 'textdomain'),
        'menu_name'          => _x('Shortlinks', 'admin menu', 'textdomain'),
        'name_admin_bar'     => _x('Shortlink', 'add new on admin bar', 'textdomain'),
        'add_new'            => _x('Add New', 'shortlink', 'textdomain'),
        'add_new_item'       => __('Add New Shortlink', 'textdomain'),
        'new_item'           => __('New Shortlink', 'textdomain'),
        'edit_item'          => __('Edit Shortlink', 'textdomain'),
        'view_item'          => __('View Shortlink', 'textdomain'),
        'all_items'          => __('All Shortlinks', 'textdomain'),
        'search_items'       => __('Search Shortlinks', 'textdomain'),
        'parent_item_colon'  => __('Parent Shortlinks:', 'textdomain'),
        'not_found'          => __('No shortlinks found.', 'textdomain'),
        'not_found_in_trash' => __('No shortlinks found in Trash.', 'textdomain'),
    );

    $args = array(
        'labels'             => $labels,
        'public'             => true,
        'publicly_queryable' => true,
        'show_ui'            => true,
        'show_in_menu'       => true,
        'query_var'          => true,
        'rewrite'            => array('slug' => ''), // Empty slug for no prefix
        'capability_type'    => 'post',
        'has_archive'        => false,
        'hierarchical'       => false,
        'menu_position'      => 20,
        'supports'           => array('title'),
    );

    register_post_type('shortlink', $args);
}

add_action('init', 'register_shortlink_post_type');

// Register Shortlink Categories (Taxonomy)
function register_shortlink_taxonomy() {
    $labels = array(
        'name'              => _x('Prefix', 'taxonomy general name', 'textdomain'),
        'singular_name'     => _x('Prefix', 'taxonomy singular name', 'textdomain'),
        'search_items'      => __('Search Prefix', 'textdomain'),
        'all_items'         => __('All Prefix', 'textdomain'),
        'parent_item'       => __('Parent Prefix', 'textdomain'),
        'parent_item_colon' => __('Parent Prefix:', 'textdomain'),
        'edit_item'         => __('Edit Prefix', 'textdomain'),
        'update_item'       => __('Update Prefix', 'textdomain'),
        'add_new_item'      => __('Add New Prefix', 'textdomain'),
        'new_item_name'     => __('New Prefix Name', 'textdomain'),
        'menu_name'         => __('Prefix', 'textdomain'),
    );

    $args = array(
        'hierarchical'      => true,
        'labels'            => $labels,
        'show_ui'           => true,
        'show_admin_column' => true,
        'query_var'         => true,
        'rewrite'           => array('slug' => 'shortlink-category'),
    );

    register_taxonomy('shortlink_category', array('shortlink'), $args);
}

add_action('init', 'register_shortlink_taxonomy');

// Add meta boxes for shortlink fields
function add_shortlink_meta_boxes() {
    add_meta_box(
        'shortlink_details',
        'Shortlink Details',
        'shortlink_meta_box_callback',
        'shortlink',
        'normal',
        'high'
    );
}

add_action('add_meta_boxes', 'add_shortlink_meta_boxes');

// Meta box callback
function shortlink_meta_box_callback($post) {
    wp_nonce_field('shortlink_save_meta_box_data', 'shortlink_meta_box_nonce');

    $redirect_url = get_post_meta($post->ID, '_redirect_url', true);
    $redirect_code = get_post_meta($post->ID, '_redirect_code', true);
    $expiry_date = get_post_meta($post->ID, '_expiry_date', true);
    $hide_prefix = get_post_meta($post->ID, '_hide_prefix', true);
    $category = wp_get_post_terms($post->ID, 'shortlink_category', array('fields' => 'ids'));

    $redirect_codes = array(
        '301' => '301 Moved Permanently',
        '302' => '302 Found',
        '307' => '307 Temporary Redirect',
        '308' => '308 Permanent Redirect',
    );

    // URL generation based on category
    $shortlink_code = get_post_field('post_name', $post->ID);
    $category_slug = !empty($category) ? get_term($category[0])->slug : '';
    $full_url = home_url('/' . ($hide_prefix ? '' : $category_slug . '/') . $shortlink_code);
    ?>
    <p>
        <label for="redirect_url">Redirect URL:</label>
        <input type="url" id="redirect_url" name="redirect_url" value="<?php echo esc_attr($redirect_url); ?>" />
    </p>
    <p>
        <label for="redirect_code">Redirect Code:</label>
        <select id="redirect_code" name="redirect_code">
            <?php foreach ($redirect_codes as $code => $description): ?>
                <option value="<?php echo esc_attr($code); ?>" <?php selected($redirect_code, $code); ?>><?php echo esc_html($description); ?></option>
            <?php endforeach; ?>
        </select>
    </p>
    <p>
        <label for="expiry_date">Expiry Date:</label>
        <input type="date" id="expiry_date" name="expiry_date" value="<?php echo esc_attr($expiry_date); ?>" />
    </p>
    <p>
        <label for="hide_prefix">
            <input type="checkbox" id="hide_prefix" name="hide_prefix" value="1" <?php checked($hide_prefix, '1'); ?> />
            <?php _e('Hide Prefix'); ?>
        </label>
    </p>
    <p>
        <label for="shortlink_category">Category:</label>
        <?php
        wp_dropdown_categories(array(
            'taxonomy' => 'shortlink_category',
            'hide_empty' => false,
            'name' => 'shortlink_category',
            'id' => 'shortlink_category',
            'selected' => !empty($category) ? $category[0] : '',
            'show_option_none' => 'Select Category'
        ));
        ?>
    </p>
    <p>
        <label for="shortlink_url">Shortlink URL:</label>
        <input type="text" id="shortlink_url" value="<?php echo esc_attr($full_url); ?>" readonly style="width: 100%; cursor: pointer;" onclick="this.select(); document.execCommand('copy'); alert('URL copied to clipboard!');" />
    </p>
    <p>
        <strong>First Click:</strong> <?php echo get_post_meta($post->ID, '_first_click', true) ? date('d.m.Y H:i', strtotime(get_post_meta($post->ID, '_first_click', true))) : 'Not Clicked Yet'; ?><br />
        <strong>Last Click:</strong> <?php echo get_post_meta($post->ID, '_last_click', true) ? date('d.m.Y H:i', strtotime(get_post_meta($post->ID, '_last_click', true))) : 'No Clicks Yet'; ?>
    </p>
    <?php
}

// Save meta box data
function save_shortlink_meta_box_data($post_id) {
    if (!isset($_POST['shortlink_meta_box_nonce'])) {
        return;
    }

    if (!wp_verify_nonce($_POST['shortlink_meta_box_nonce'], 'shortlink_save_meta_box_data')) {
        return;
    }

    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
        return;
    }

    if (!current_user_can('edit_post', $post_id)) {
        return;
    }

    $redirect_url = esc_url_raw($_POST['redirect_url']);
    $redirect_code = sanitize_text_field($_POST['redirect_code']);
    $expiry_date = sanitize_text_field($_POST['expiry_date']);
    $hide_prefix = isset($_POST['hide_prefix']) ? '1' : '';
    $category_id = intval($_POST['shortlink_category']);

    update_post_meta($post_id, '_redirect_url', $redirect_url);
    update_post_meta($post_id, '_redirect_code', $redirect_code);
    update_post_meta($post_id, '_expiry_date', $expiry_date);
    update_post_meta($post_id, '_hide_prefix', $hide_prefix);
    wp_set_post_terms($post_id, array($category_id), 'shortlink_category');
}

add_action('save_post', 'save_shortlink_meta_box_data');

// Handle redirects
function handle_shortlink_redirect() {
    if (is_singular('shortlink')) {
        global $post;
        $redirect_url = get_post_meta($post->ID, '_redirect_url', true);
        $redirect_code = get_post_meta($post->ID, '_redirect_code', true);

        if ($redirect_url) {
            // Update clicks and timestamps
            $clicks = intval(get_post_meta($post->ID, '_clicks', true)) + 1;
            update_post_meta($post->ID, '_clicks', $clicks);
            update_post_meta($post->ID, '_last_click', current_time('mysql'));

            if (empty(get_post_meta($post->ID, '_first_click', true))) {
                update_post_meta($post->ID, '_first_click', current_time('mysql'));
            }

            wp_redirect($redirect_url, $redirect_code);
            exit;
        }
    }
}

add_action('template_redirect', 'handle_shortlink_redirect');

// Custom column management
function add_custom_columns($columns) {
    $columns['shortlink'] = __('Shortlink', 'textdomain');
    $columns['redirect_url'] = __('Redirect URL', 'textdomain');
    $columns['clicks'] = __('Clicks', 'textdomain');
    $columns['first_click'] = __('First Click', 'textdomain');
    $columns['last_click'] = __('Last Click', 'textdomain');

    return $columns;
}

add_filter('manage_shortlink_posts_columns', 'add_custom_columns');

function custom_column_content($column_name, $post_id) {
    switch ($column_name) {
        case 'shortlink':
            $shortlink_code = get_post_field('post_name', $post_id);
            $hide_prefix = get_post_meta($post_id, '_hide_prefix', true);
            $category = wp_get_post_terms($post_id, 'shortlink_category', array('fields' => 'slugs'));
            $category_slug = !empty($category) ? $category[0] : '';
            $full_url = home_url('/' . ($hide_prefix ? '' : $category_slug . '/') . $shortlink_code);
            echo '<input type="text" value="' . esc_attr($full_url) . '" readonly style="width: 100%; cursor: pointer;" onclick="this.select(); document.execCommand(\'copy\'); alert(\'URL copied to clipboard!\');" />';
            break;
        case 'redirect_url':
            echo get_post_meta($post_id, '_redirect_url', true);
            break;
        case 'clicks':
            echo get_post_meta($post_id, '_clicks', true);
            break;
        case 'first_click':
            $first_click = get_post_meta($post_id, '_first_click', true);
            echo $first_click ? human_time_diff(strtotime($first_click), current_time('timestamp')) . ' ago' : 'Not Clicked Yet';
            break;
        case 'last_click':
            $last_click = get_post_meta($post_id, '_last_click', true);
            echo $last_click ? human_time_diff(strtotime($last_click), current_time('timestamp')) . ' ago' : 'No Clicks Yet';
            break;
    }
}

add_action('manage_shortlink_posts_custom_column', 'custom_column_content', 10, 2);

// Update rewrite rules for categories
function update_rewrite_rules() {
    $categories = get_terms(array(
        'taxonomy' => 'shortlink_category',
        'hide_empty' => false,
    ));
    foreach ($categories as $category) {
        add_rewrite_rule(
            '^' . $category->slug . '/([^/]+)/?$',
            'index.php?post_type=shortlink&shortlink_category=' . $category->slug . '&name=$matches[1]',
            'top'
        );
    }
    add_rewrite_rule('^([^/]+)/?$', 'index.php?post_type=shortlink&name=$matches[1]', 'top');
}

add_action('init', 'update_rewrite_rules', 10, 0);

// Update settings link
function shortlink_settings_link($links) {
    $settings_link = '<a href="options-general.php?page=shortlink_settings">Settings</a>';
    array_unshift($links, $settings_link);
    return $links;
}

add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'shortlink_settings_link');

// Add settings page
function shortlink_settings_menu() {
    add_options_page(
        'Shortlink Settings',
        'Shortlink Settings',
        'manage_options',
        'shortlink_settings',
        'shortlink_settings_page'
    );
}

add_action('admin_menu', 'shortlink_settings_menu');

function shortlink_settings_page() {
    ?>
    <div class="wrap">
        <h1>Shortlink Settings</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('shortlink_options');
            do_settings_sections('shortlink_settings');
            submit_button();
            ?>
        </form>
    </div>
    <?php
}

// Register settings
function shortlink_register_settings() {
    register_setting('shortlink_options', 'shortlink_prefix');
}

add_action('admin_init', 'shortlink_register_settings');
